@@ -1,26 +1,14 @@
 use super::super::fixed_bytes_str::four_bytes::CustomString;
-use super::trie_custom::Trie;
-use rayon::prelude::*;
+use super::trie_char_ver::TrieChar as Trie;
 use std::io::BufReader;
 use std::{error::Error, io::prelude::*};
 use std::{fs::File, path::PathBuf};
 
-const DEFAULT_DICT_FILE: &str = include_str!("../../words_th.txt");
-
 pub enum DictSource {
     WordList(Vec<String>),
     FilePath(PathBuf),
 }
 
-pub fn create_default_dict() -> Trie {
-    let default_dict = DEFAULT_DICT_FILE
-        .par_lines()
-        .map(|word| CustomString::new(word))
-        .filter(|word| !word.is_empty())
-        .collect::<Vec<CustomString>>();
-    Trie::new(&default_dict)
-}
-
 pub fn create_dict_trie(source: DictSource) -> Result<Trie, Box<dyn Error>> {
     match source {
         DictSource::FilePath(single_source) => {
