@@ -11,23 +11,26 @@ with heuristic graph size limit added to avoid exponential wait time.
 
 Rust implementation: ["Thanathip Suntorntip"]
 */
-// TODO: use slice_by_chars_indice on &[u8]
+use super::super::fixed_bytes_str::four_bytes::{
+    CustomStringBytesSlice, CustomStringBytesVec, FixedLengthCustomString,
+};
+use super::{
+    dict_reader_custom::{create_dict_trie, DictSource},
+    tcc_custom,
+    tokenizer_trait::Tokenizer,
+    trie_char_ver::TrieChar as Trie,
+};
 use crate::fixed_bytes_str::four_bytes::{
     rfind_space_char_index, CustomString, FixedCharsLengthByteSlice, BYTES_PER_CHAR,
 };
-
-use super::super::fixed_bytes_str::four_bytes::{CustomStringBytesSlice, CustomStringBytesVec};
-use super::{
-    dict_reader_custom::{create_default_dict, create_dict_trie, DictSource},
-    tcc_custom,
-    tokenizer_trait::Tokenizer,
-    trie_custom::{Trie},
-};
-use ahash::{AHashMap as HashMap, AHashSet as HashSet};
+use anyhow::Result as AnyResult;
 use binary_heap_plus::{BinaryHeap, MinComparator};
 use lazy_static::lazy_static;
 use rayon::prelude::*;
 use regex::bytes::Regex;
+use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
+use std::error::Error;
+use std::fmt::Display;
 use std::{collections::VecDeque, path::PathBuf};
 const MAX_GRAPH_SIZE: usize = 50;
 const USE_MULTITHREAD_THRESHOLD: usize = 10000;
@@ -63,30 +66,53 @@ lazy_static! {
     static ref THAI_TWOCHARS_PATTERN: Regex = Regex::new(r"^(\x00[ก-ฮ]){0,2}$").unwrap();
 }
 
+#[derive(Clone, Debug)]
+struct BFSSearchError {
+    graph: HashMap<CharacterIndex, Vec<CharacterIndex>>,
+    start: CharacterIndex,
+    goal: CharacterIndex,
+}
+impl BFSSearchError {
+    pub fn new(
+        graph: &HashMap<CharacterIndex, Vec<CharacterIndex>>,
+        start: CharacterIndex,
+        goal: CharacterIndex,
+    ) -> Self {
+        Self {
+            graph: graph.clone(),
+            start,
+            goal,
+        }
+    }
+}
+impl Display for BFSSearchError {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(
+            f,
+            "Cannot find goal position {} with start position {} with graph {:?}",
+            self.goal, self.start, self.goal
+        )
+    }
+}
+impl Error for BFSSearchError {}
+#[derive(Debug)]
 pub struct Newmm {
     dict: Box<Trie>,
 }
-
 impl Newmm {
-    pub fn new(dict_path: Option<&str>) -> Self {
-        match dict_path {
-            None => Self {
-                dict: Box::from(create_default_dict()),
-            },
-            Some(path) => Self {
-                dict: Box::from(
-                    create_dict_trie(DictSource::FilePath(PathBuf::from(path))).unwrap(),
-                ),
-            },
+    pub fn new(dict_path: &str) -> Self {
+        Self {
+            dict: Box::from(
+                create_dict_trie(DictSource::FilePath(PathBuf::from(dict_path))).unwrap(),
+            ),
         }
     }
-
     fn bfs_paths_graph(
         graph: &HashMap<CharacterIndex, Vec<CharacterIndex>>,
         start: CharacterIndex,
         goal: CharacterIndex,
         current_queue: &mut VecDeque<(usize, Vec<usize>)>,
-    ) -> Vec<CharacterIndex> {
+    ) -> AnyResult<Vec<CharacterIndex>> {
         current_queue.clear();
 
         // let mut current_queue: VecDeque<(usize, Vec<usize>)> = VecDeque::with_capacity(graph.len());
@@ -94,8 +120,8 @@ impl Newmm {
         let mut init_path: Vec<usize> = Vec::with_capacity(goal - start);
         init_path.push(start);
         current_queue.push_back((start, init_path));
-        while !current_queue.is_empty() {
-            let (vertex, path) = current_queue.pop_front().unwrap();
+
+        while let Some((vertex, path)) = current_queue.pop_front() {
             if let Some(idk) = graph.get(&vertex) {
                 for position in idk {
                     if *position != goal {
@@ -106,31 +132,33 @@ impl Newmm {
                         let mut appended_path = path;
                         appended_path.push(*position);
 
-                        return appended_path;
+                        return Ok(appended_path);
                     };
                 }
             };
         }
-        panic!("something wrong");
+
+        Err(BFSSearchError::new(graph, start, goal).into())
     }
 
     fn one_cut<'a, 'b>(
-        input: &'a CustomStringBytesSlice,
+        input: &'a CustomString,
         custom_dict: &'b Trie,
-    ) -> Vec<&'a CustomStringBytesSlice> {
+    ) -> AnyResult<Vec<&'a CustomStringBytesSlice>> {
         let text = input;
         let input_char_len = text.chars_len();
         let mut reused_queue: VecDeque<(usize, Vec<usize>)> = VecDeque::with_capacity(10);
         let mut graph_size: usize = 0;
-        let mut graph: HashMap<CharacterIndex, Vec<CharacterIndex>> =
-            HashMap::with_capacity(input_char_len / 100);
-        let mut result_str: Vec<&CustomStringBytesSlice> = Vec::with_capacity(input_char_len / 100);
+        let mut graph: HashMap<CharacterIndex, Vec<CharacterIndex>> = HashMap::default();
+        graph.reserve(input_char_len / 10);
+        let mut result_str: Vec<&CustomStringBytesSlice> = Vec::with_capacity(input_char_len / 10);
 
         // all position should be refered as character index
-        let valid_position = tcc_custom::tcc_pos(input);
+        let valid_position = tcc_custom::tcc_pos(text.raw_content());
         let text_length = input_char_len;
         let mut position_list: BinaryHeap<CharacterIndex, MinComparator> = BinaryHeap::new_min();
-        let mut existing_candidate: HashSet<CharacterIndex> = HashSet::with_capacity(50);
+        let mut existing_candidate: HashSet<CharacterIndex> = HashSet::default();
+        existing_candidate.reserve(input_char_len / 10);
         position_list.push(0);
         existing_candidate.insert(0);
         let mut end_position: CharacterIndex = 0;
@@ -142,8 +170,8 @@ impl Newmm {
             _ => false,
         } {
             if let Some(begin_position) = position_list.pop() {
-                let sub_text_prefix = text.slice_by_char_indice(begin_position, text.chars_len());
-                let prefixes = Trie::prefix_ref(sub_text_prefix, &custom_dict);
+                let sub_text_prefix = text.substring(begin_position, text.chars_len());
+                let prefixes = Trie::prefix_ref(&sub_text_prefix, custom_dict);
                 for word in prefixes {
                     let word_length = word.chars_len();
                     let end_position_candidate = begin_position + word_length;
@@ -177,13 +205,13 @@ impl Newmm {
                             end_position,
                             *first_position_list,
                             &mut reused_queue,
-                        );
+                        )?;
                         graph_size = 0; // reset our graph
 
                         for position in group_of_end_position_candidate.iter().skip(1) {
-                            let token = text.slice_by_char_indice(end_position, *position);
+                            let token_bytes = text.substring_as_bytes(end_position, *position);
 
-                            result_str.push(token);
+                            result_str.push(token_bytes);
                             end_position = *position;
                         }
                     } else {
@@ -191,13 +219,14 @@ impl Newmm {
                     }
                 } else if position_list_length == 0 {
                     // no candidate, deal with non-dict word
-                    match NON_THAI_PATTERN.find(sub_text_prefix) {
+                    match NON_THAI_PATTERN.find(sub_text_prefix.raw_content()) {
                         Some(match_point) => {
                             let matched_start_char_index = match_point.start() / BYTES_PER_CHAR;
                             let matched_end_char_index = match_point.end() / BYTES_PER_CHAR;
                             //  non thai -> skip to the end of match
                             end_position = begin_position
                                 + sub_text_prefix
+                                    .raw_content()
                                     .slice_by_char_indice(
                                         matched_start_char_index,
                                         matched_end_char_index,
@@ -209,14 +238,14 @@ impl Newmm {
                             let mut finish_without_break = true;
                             for position in begin_position + 1..text_length {
                                 if valid_position.contains(&position) {
-                                    let prefix = &text.slice_by_char_indice(position, text_length);
+                                    let prefix = text.substring(position, text_length);
 
-                                    let list_of_prefixes = Trie::prefix_ref(prefix, &custom_dict);
+                                    let list_of_prefixes = Trie::prefix_ref(&prefix, custom_dict);
                                     let valid_word_filter = |word: &&[u8]| {
                                         let new_position = position + word.chars_len();
                                         let is_valid = valid_position.contains(&new_position);
                                         let is_two_thai_chars =
-                                            THAI_TWOCHARS_PATTERN.is_match(&word);
+                                            THAI_TWOCHARS_PATTERN.is_match(word);
                                         is_valid && !is_two_thai_chars
                                     };
                                     let valid_words: Vec<&[u8]> =
@@ -237,7 +266,7 @@ impl Newmm {
                                         finish_without_break = false;
                                         break;
                                     };
-                                    if NON_THAI_PATTERN.is_match(&prefix) {
+                                    if NON_THAI_PATTERN.is_match(prefix.raw_content()) {
                                         end_position = position;
                                         finish_without_break = false;
                                         break;
@@ -249,74 +278,70 @@ impl Newmm {
                             }
                         }
                     }
-                    let current_graph_opt = graph.get_mut(&begin_position);
-                    match current_graph_opt {
-                        Some(existing_path) => {
-                            existing_path.push(end_position);
-                            graph_size += 1;
-                            let token = text.slice_by_char_indice(begin_position, end_position);
+                    if let Some(existing_path) = graph.get_mut(&begin_position) {
+                        existing_path.push(end_position);
+                        graph_size += 1;
+                        let token_bytes = text.substring_as_bytes(begin_position, end_position);
 
-                            result_str.push(token);
-                            position_list.push(end_position);
-                            existing_candidate.insert(end_position);
-                        }
-                        None => {
-                            let mut graph_elem: Vec<usize> = Vec::with_capacity(10);
-                            graph_elem.push(end_position);
-                            graph.insert(begin_position, graph_elem);
-                            graph_size += 1;
-                            let token = text.slice_by_char_indice(begin_position, end_position);
-                            result_str.push(token);
-                            position_list.push(end_position);
-                            existing_candidate.insert(end_position);
-                        }
+                        result_str.push(token_bytes);
+                        position_list.push(end_position);
+                        existing_candidate.insert(end_position);
+                    } else {
+                        let mut graph_elem: Vec<usize> = Vec::with_capacity(10);
+                        graph_elem.push(end_position);
+                        graph.insert(begin_position, graph_elem);
+                        graph_size += 1;
+                        let token_bytes = text.substring_as_bytes(begin_position, end_position);
+                        result_str.push(token_bytes);
+                        position_list.push(end_position);
+                        existing_candidate.insert(end_position);
                     }
                 }
             }
         }
-        result_str
+        Ok(result_str)
     }
 
-    pub fn internal_segment(
+    fn internal_segment(
         input: &CustomString,
         custom_dict: &Trie,
         safe: bool,
         parallel: bool,
-    ) -> Vec<String> {
-        if input.len() == 0 {
-            return vec![];
+    ) -> AnyResult<Vec<String>> {
+        if input.is_empty() {
+            return Ok(vec![]);
         }
         if !safe || input.chars_len() < TEXT_SCAN_END {
-            let result = Self::one_cut(input.raw_content(), custom_dict);
-            if parallel {
+            let result = Self::one_cut(input, custom_dict)?;
+            Ok(if parallel {
                 result
                     .into_par_iter()
-                    .map(|custom_string_bytes| {
-                        CustomString::convert_raw_bytes_to_std_string(&custom_string_bytes)
+                    .map(|custom_substring| {
+                        CustomString::convert_raw_bytes_to_std_string(custom_substring)
                     })
                     .collect()
             } else {
                 result
                     .into_iter()
-                    .map(|custom_string_bytes| {
-                        CustomString::convert_raw_bytes_to_std_string(&custom_string_bytes)
+                    .map(|custom_substring| {
+                        CustomString::convert_raw_bytes_to_std_string(custom_substring)
                     })
                     .collect()
-            }
+            })
         } else {
-            let mut txt = input.raw_content();
-            let mut txt_parts: Vec<CustomStringBytesVec> = Vec::with_capacity(txt.len() / 10);
+            let mut txt = input.substring(0, input.chars_len());
+            let mut txt_parts: Vec<CustomString> = Vec::with_capacity(txt.chars_len() / 10);
             while txt.chars_len() >= TEXT_SCAN_END {
-                let sample: &[u8] = txt.slice_by_char_indice(TEXT_SCAN_BEGIN, TEXT_SCAN_END);
+                let sample = txt.substring(TEXT_SCAN_BEGIN, TEXT_SCAN_END);
 
                 let mut cut_pos;
 
-                let space_char_index = rfind_space_char_index(sample);
+                let space_char_index = rfind_space_char_index(sample.raw_content());
                 // there is a space
                 if let Some(space_char_index) = space_char_index {
                     cut_pos = space_char_index + 1;
                 } else {
-                    let word_tokens = Self::one_cut(sample, &custom_dict);
+                    let word_tokens = Self::one_cut(&sample, custom_dict)?;
                     let mut token_max_index = 0;
                     let mut token_max_length = 0;
                     for (idx, token) in word_tokens.iter().enumerate() {
@@ -331,55 +356,49 @@ impl Newmm {
                         cut_pos = cut_pos + word_tokens.get(i).unwrap().chars_len();
                     }
                 }
-                txt_parts.push(txt.slice_by_char_indice(0, cut_pos).to_owned());
-                txt = txt.slice_by_char_indice(cut_pos, txt.chars_len());
+                txt_parts.push(txt.substring(0, cut_pos));
+                txt = txt.substring(cut_pos, txt.chars_len());
             }
             if !txt.is_empty() {
-                txt_parts.push(txt.to_owned());
+                txt_parts.push(txt);
             }
 
-            if parallel {
+            Ok(if parallel {
                 txt_parts
-                    .into_par_iter()
-                    .flat_map(|part| {
-                        Self::one_cut(&part, &custom_dict)
+                    .par_iter()
+                    .flat_map(|part| -> AnyResult<_> {
+                        let bind_part = &part.substring(0, part.chars_len());
+                        let words = Self::one_cut(bind_part, custom_dict)?;
+                        Ok(words
                             .into_par_iter()
-                            .map(|word| CustomString::convert_raw_bytes_to_std_string(&word))
-                            .collect::<Vec<String>>()
+                            .map(|word| CustomString::convert_raw_bytes_to_std_string(word))
+                            .collect::<Vec<String>>())
                     })
-                    .collect::<Vec<String>>()
+                    .flatten()
+                    .collect()
             } else {
                 txt_parts
                     .iter()
-                    .flat_map(|part| {
-                        Self::one_cut(&part, &custom_dict)
-                            .iter()
-                            .map(|word| CustomString::convert_raw_bytes_to_std_string(&word))
-                            .collect::<Vec<String>>()
+                    .flat_map(|part| -> AnyResult<_> {
+                        Ok(
+                            Self::one_cut(&part.substring(0, part.chars_len()), custom_dict)?
+                                .iter()
+                                .map(|word| CustomString::convert_raw_bytes_to_std_string(word))
+                                .collect::<Vec<String>>(),
+                        )
                     })
-                    .collect::<Vec<String>>()
-            }
+                    .flatten()
+                    .collect()
+            })
         }
     }
 }
-
 impl Tokenizer for Newmm {
-    fn segment(&self, text: &str, safe: Option<bool>, parallel: Option<bool>) -> Vec<String> {
-        let safe_flag = safe.unwrap_or(false);
-        let parallel_flag = match parallel {
-            Some(val) => val,
-            _ => false,
-        };
-        let custom_string = CustomString::new(text);
-        Self::internal_segment(&custom_string, &self.dict, safe_flag, parallel_flag)
+    fn segment(&self, text: &str, safe: bool, parallel: bool) -> AnyResult<Vec<String>> {
+        Self::internal_segment(&CustomString::new(text), &self.dict, safe, parallel)
     }
 
-    fn segment_to_string(
-        &self,
-        text: &str,
-        safe: Option<bool>,
-        parallel: Option<bool>,
-    ) -> Vec<String> {
-        self.segment(text, safe, parallel)
+    fn segment_to_string(&self, text: &str, safe: bool, parallel: bool) -> Vec<String> {
+        self.segment(text, safe, parallel).unwrap()
     }
 }
